<!DOCTYPE html>
<html lang="en">

<head>
  <meta charset="utf-8" />
  <link rel="icon" href="{{ url_for('public', path='favicon.ico') }}" />
  <meta name="viewport" content="width=device-width, initial-scale=1" />
  <meta name="theme-color" content="#000000" />

  <meta name="description" content="FastAPI App generated by pd">
  <meta name="author" content="pd">

  <link rel="apple-touch-icon" href="{{ url_for('public', path='logo192.png') }}" />

  <link rel="manifest" href="{{ url_for('public', path='manifest.json') }}" />

  <title>FastAPI App</title>
</head>

<body>
  <div id="react-root">
    <!-- React will render here -->
  </div>

  <script>
    // Directly set window.DATA to the JSON-stringified data
    window.DATA = {{ data | tojson | safe }};
  </script>

  <!-- Webpack our TSX code into the bundled JS that's included below -->
  <script src="{{ url_for('public', path='js/bundle.js') }}"></script>
</body>

</html>