metadata = {
    "/": {
        "title": "Pegasus - Next Generation Web Framework | Home",
        "description": "Pegasus combines the best of both worlds, a modern web framework with a powerful backend",
        "keywords": "Pegasus, Web, Python, TypeScript",
        "robots": "index, follow",
        "og_type": "website",
        "twitter_card": "summary_large_image",
        "author": "Zeff Muks",
    },
    "/error": {
        "robots": "noindex, nofollow",
        "title": "Pegasus - Error",
        "description": "An error occurred",
    }
}
