<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="utf-8"/>
    <link rel="icon" href="%PUBLIC_URL%/favicon.ico"/>
    <meta name="viewport" content="width=device-width, initial-scale=1"/>
    <meta name="theme-color" content="#000000"/>

    <meta name="description" content="React App generated by pd">
    <meta name="author" content="pd">

    <link rel="apple-touch-icon" href="%PUBLIC_URL%/logo192.png"/>

    <link rel="manifest" href="%PUBLIC_URL%/manifest.json"/>

    <title>React App</title>
</head>
<body>
<noscript>You need to enable JavaScript to run this app.</noscript>
<div id="root"></div>
<!--This is where React will render the app.-->
</body>
</html>
