import '@/app/ui/global.css'
import {rosario} from '@/app/ui/fonts';
import type {Metadata} from "next";

export const metadata: Metadata = {
    title: "NextJS App",
    description: "NextJS App generated by pd",
};

export default function RootLayout({children}: Readonly<{
    children: React.ReactNode;
}>) {
    return (
        <html lang="en">
        <body className={rosario.className}>{children}</body>
        </html>
    );
}
