import Image from "next/image";

export default function Home() {
    return (
        <main>
            <p>Hello World from <span className="text-[#0000ff]">pd</span></p>

            <Image src={"/pd.png"} alt={"/pd"} width="426" height="261"/>

            <footer>
                <p>Generated by pd © 2023. All Rights Reserved.</p>
            </footer>
        </main>
    );
}
