/** @type {import('tailwindcss').Config} */
module.exports = {
    content: [
        "./ui/**/*.{js,jsx,ts,tsx}",
        "./templates/**/*.html",
    ],
    theme: {
        extend: {},
    },
    plugins: [],
}